@@ -25,7 +25,11 @@
 #define LIBPANEL_INSIDE
 # include "panel-types.h"
 # include "panel-enums.h"
+# include "panel-action-group.h"
+# include "panel-action-muxer.h"
+# include "panel-application.h"
 # include "panel-dock.h"
+# include "panel-document-workspace.h"
 # include "panel-frame.h"
 # include "panel-frame-header.h"
 # include "panel-frame-header-bar.h"
@@ -33,14 +37,21 @@
 # include "panel-frame-tab-bar.h"
 # include "panel-grid.h"
 # include "panel-grid-column.h"
+# include "panel-gsettings-action-group.h"
 # include "panel-init.h"
+# include "panel-inhibitor.h"
+# include "panel-layered-settings.h"
+# include "panel-menu-manager.h"
 # include "panel-omni-bar.h"
 # include "panel-paned.h"
 # include "panel-position.h"
 # include "panel-save-delegate.h"
 # include "panel-save-dialog.h"
+# include "panel-session.h"
+# include "panel-session-item.h"
 # include "panel-statusbar.h"
 # include "panel-theme-selector.h"
 # include "panel-toggle-button.h"
 # include "panel-widget.h"
+# include "panel-workbench.h"
 #undef LIBPANEL_INSIDE
