@@ -69,6 +69,8 @@ typedef struct
 } PanelDockPrivate;
 
 static void buildable_iface_init (GtkBuildableIface *iface);
+static GtkWidget *panel_dock_get_child_for_area (PanelDock *self,
+                                                 PanelArea  area);
 
 G_DEFINE_TYPE_WITH_CODE (PanelDock, panel_dock, GTK_TYPE_WIDGET,
                          G_ADD_PRIVATE (PanelDock)
@@ -741,6 +743,50 @@ panel_dock_notify_empty_cb (PanelDock      *self,
     }
 }
 
+static void
+panel_dock_notify_reveal_child_cb (PanelDock      *self,
+                                   GParamSpec     *pspec,
+                                   PanelDockChild *child)
+{
+  PanelArea area;
+  gboolean reveal_child;
+  const char *css_name;
+
+  g_assert (PANEL_IS_DOCK (self));
+  g_assert (PANEL_IS_DOCK_CHILD (child));
+
+  area = panel_dock_child_get_area (child);
+  reveal_child = panel_dock_child_get_reveal_child (child);
+
+  switch (area)
+    {
+    case PANEL_AREA_START:
+      css_name = "start-revealed";
+      break;
+
+    case PANEL_AREA_END:
+      css_name = "end-revealed";
+      break;
+
+    case PANEL_AREA_TOP:
+      css_name = "top-revealed";
+      break;
+
+    case PANEL_AREA_BOTTOM:
+      css_name = "bottom-revealed";
+      break;
+
+    case PANEL_AREA_CENTER:
+    default:
+      return;
+    }
+
+  if (reveal_child)
+    gtk_widget_add_css_class (GTK_WIDGET (self), css_name);
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (self), css_name);
+}
+
 static GtkWidget *
 get_or_create_dock_child (PanelDock *self,
                           PanelArea  area,
@@ -766,13 +812,20 @@ get_or_create_dock_child (PanelDock *self,
     }
 
   child = panel_dock_child_new (area);
-  panel_dock_child_set_reveal_child (PANEL_DOCK_CHILD (child), FALSE);
+  panel_dock_child_set_reveal_child (PANEL_DOCK_CHILD (child), area == PANEL_AREA_CENTER);
+  g_signal_connect_object (child,
+                           "notify::reveal-child",
+                           G_CALLBACK (panel_dock_notify_reveal_child_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
   g_signal_connect_object (child,
                            "notify::empty",
                            G_CALLBACK (panel_dock_notify_empty_cb),
                            self,
                            G_CONNECT_SWAPPED);
   gtk_grid_attach (priv->grid, child, left, top, width, height);
+  panel_dock_notify_empty_cb (self, NULL, PANEL_DOCK_CHILD (child));
+  panel_dock_notify_reveal_child_cb (self, NULL, PANEL_DOCK_CHILD (child));
 
   return child;
 }
@@ -898,17 +951,58 @@ panel_dock_add_child (GtkBuildable *buildable,
     {
       if (drag_position != -1)
         panel_dock_child_set_drag_position (PANEL_DOCK_CHILD (object), drag_position);
+      g_signal_connect_object (object,
+                               "notify::empty",
+                               G_CALLBACK (panel_dock_notify_empty_cb),
+                               self,
+                               G_CONNECT_SWAPPED);
+      g_signal_connect_object (object,
+                               "notify::reveal-child",
+                               G_CALLBACK (panel_dock_notify_reveal_child_cb),
+                               self,
+                               G_CONNECT_SWAPPED);
       gtk_grid_attach (priv->grid, GTK_WIDGET (object), left, top, width, height);
+      panel_dock_notify_empty_cb (self, NULL, PANEL_DOCK_CHILD (object));
+      panel_dock_notify_reveal_child_cb (self, NULL, PANEL_DOCK_CHILD (object));
     }
 
   notify_can_reveal (self, area);
   set_reveal (self, area, reveal);
 }
 
+static GObject *
+panel_dock_get_internal_child (GtkBuildable *buildable,
+                               GtkBuilder   *builder,
+                               const char   *name)
+{
+  PanelDock *self = PANEL_DOCK (buildable);
+  PanelArea area = 0;
+  GtkOrientation orientation;
+  int left, top, width, height;
+
+  if (g_strcmp0 (name, "start") == 0)
+    area = PANEL_AREA_START;
+  else if (g_strcmp0 (name, "end") == 0)
+    area = PANEL_AREA_END;
+  else if (g_strcmp0 (name, "top") == 0)
+    area = PANEL_AREA_TOP;
+  else if (g_strcmp0 (name, "bottom") == 0)
+    area = PANEL_AREA_BOTTOM;
+  else if (g_strcmp0 (name, "center") == 0)
+    area = PANEL_AREA_CENTER;
+  else
+    return NULL;
+
+  get_grid_positions (area, &left, &top, &width, &height, &orientation);
+
+  return G_OBJECT (get_or_create_dock_child (self, area, left, top, width, height));
+}
+
 static void
 buildable_iface_init (GtkBuildableIface *iface)
 {
   iface->add_child = panel_dock_add_child;
+  iface->get_internal_child = panel_dock_get_internal_child;
 }
 
 /**
@@ -1343,6 +1437,11 @@ _panel_dock_end_drag (PanelDock   *self,
   unprepare_from_drag (self, PANEL_AREA_END);
   unprepare_from_drag (self, PANEL_AREA_TOP);
   unprepare_from_drag (self, PANEL_AREA_BOTTOM);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_REVEAL_TOP]);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_REVEAL_BOTTOM]);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_REVEAL_START]);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_REVEAL_END]);
 }
 
 void
